package com.codeborne.selenide.webdriver;

import com.codeborne.selenide.Browser;
import com.codeborne.selenide.SelenideConfig;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Test;
import org.openqa.selenium.Capabilities;
import org.openqa.selenium.Proxy;
import org.openqa.selenium.firefox.FirefoxOptions;
import org.openqa.selenium.firefox.FirefoxProfile;

import java.io.File;
import java.nio.file.Paths;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import static com.codeborne.selenide.webdriver.SeleniumCapabilitiesHelper.getBrowserLaunchArgs;
import static com.codeborne.selenide.webdriver.SeleniumCapabilitiesHelper.getBrowserLaunchPrefs;
import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.mock;

final class FirefoxDriverFactoryTest {
  private static final String DOWNLOADS_FOLDER = Paths.get("blah", "downloads").toString();

  private final Proxy proxy = mock(Proxy.class);
  private final FirefoxDriverFactory driverFactory = new FirefoxDriverFactory();
  private final SelenideConfig config = new SelenideConfig().downloadsFolder("build/should-not-be-used");
  private final File browserDownloadsFolder = new File(DOWNLOADS_FOLDER).getAbsoluteFile();
  private final Browser browser = new Browser(config.browser(), config.headless());
  private final Set<String> systemProperties = new HashSet<>();

  @AfterEach
  void tearDown() {
    for (String name : systemProperties) {
      System.clearProperty(name);
    }
  }

  private void givenSystemProperty(String name, String value) {
    systemProperties.add(name);
    System.setProperty(name, value);
  }

  @Test
  void transfersStringCapabilitiesFromSystemPropsToDriver() {
    givenSystemProperty("capabilities.some.cap", "abcd");
    assertThat(driverFactory.createCommonCapabilities(config, browser, proxy).getCapability("some.cap")).isEqualTo("abcd");
  }

  @Test
  void transfersBooleanCapabilitiesFromSystemPropsToDriver() {
    givenSystemProperty("capabilities.some.cap", "true");
    assertThat(driverFactory.createCommonCapabilities(config, browser, proxy).getCapability("some.cap"))
      .isEqualTo(true);
  }

  @Test
  void transfersIntegerCapabilitiesFromSystemPropsToDriver() {
    givenSystemProperty("capabilities.some.cap", "25");
    assertThat(driverFactory.createCommonCapabilities(config, browser, proxy).getCapability("some.cap")).isEqualTo(25);
  }

  @Test
  void transferIntegerFirefoxProfilePreferencesFromSystemPropsToDriver() {
    givenSystemProperty("firefoxprofile.some.cap", "25");
    FirefoxProfile profile = driverFactory.transferFirefoxProfileFromSystemProperties(new FirefoxOptions()).get();
    assertThat(profile.getIntegerPreference("some.cap", 0)).isEqualTo(25);
  }

  @Test
  void transferBooleanFirefoxProfilePreferencesFromSystemPropsToDriver() {
    givenSystemProperty("firefoxprofile.some.cap1", "faLSe");
    givenSystemProperty("firefoxprofile.some.cap2", "TRue");
    FirefoxProfile profile = driverFactory.transferFirefoxProfileFromSystemProperties(new FirefoxOptions()).get();
    assertThat(profile.getBooleanPreference("some.cap1", true)).isEqualTo(false);
    assertThat(profile.getBooleanPreference("some.cap2", false)).isEqualTo(true);
  }

  @Test
  void transferStringFirefoxProfilePreferencesFromSystemPropsToDriver() {
    givenSystemProperty("firefoxprofile.some.cap", "abdd");
    FirefoxProfile profile = driverFactory.transferFirefoxProfileFromSystemProperties(new FirefoxOptions()).get();
    assertThat(profile.getStringPreference("some.cap", "sjlj")).isEqualTo("abdd");
  }

  @Test
  void browserBinaryCanBeSet() {
    config.browserBinary("c:/browser.exe");
    Capabilities caps = driverFactory.createCapabilities(config, browser, proxy, browserDownloadsFolder);
    Map options = (Map) caps.asMap().get(FirefoxOptions.FIREFOX_OPTIONS);
    assertThat(options.get("binary")).isEqualTo("c:/browser.exe");
  }

  @Test
  void headlessCanBeSet() {
    config.headless(true);
    FirefoxOptions options = driverFactory.createCapabilities(config, browser, proxy, browserDownloadsFolder);
    List<String> optionArguments = getBrowserLaunchArgs(FirefoxOptions.FIREFOX_OPTIONS, options);
    assertThat(optionArguments).contains("-headless");
  }

  @Test
  void enablesProxyForLocalAddresses() {
    FirefoxOptions options = driverFactory.createCapabilities(config, browser, proxy, browserDownloadsFolder);

    Map<String, Object> prefs = prefs(options);
    assertThat(prefs.get("network.proxy.no_proxies_on")).isEqualTo("");
    assertThat(prefs.get("network.proxy.allow_hijacking_localhost")).isEqualTo(true);
    assertThat(options.asMap().get("firefox_profile")).isNull();
  }

  @Test
  void downloadsAllPopularContentTypesWithoutDialog() {
    assertThat(driverFactory.popularContentTypes()).contains(";application/pdf;");
    assertThat(driverFactory.popularContentTypes()).contains(";application/octet-stream;");
    assertThat(driverFactory.popularContentTypes()).contains(";binary/octet-stream;");
    assertThat(driverFactory.popularContentTypes()).contains(";application/msword;");
    assertThat(driverFactory.popularContentTypes()).contains(";application/vnd.ms-excel;");
    assertThat(driverFactory.popularContentTypes()).contains(";application/zip;");
    assertThat(driverFactory.popularContentTypes()).contains(";text/csv;");
  }

  @Test
  void configuresDownloadFolder() {
    config.headless(true);

    FirefoxOptions options = driverFactory.createCapabilities(config, browser, proxy, browserDownloadsFolder);

    Map<String, Object> prefs = prefs(options);
    assertThat(prefs.get("browser.download.dir")).isEqualTo(new File(DOWNLOADS_FOLDER).getAbsolutePath());
    assertThat((String) prefs.get("browser.helperApps.neverAsk.saveToDisk")).contains("application/pdf");
    assertThat(prefs.get("pdfjs.disabled")).isEqualTo(true);
    assertThat(prefs.get("browser.download.folderList")).isEqualTo(2);
  }

  @Test
  void shouldNotSetupDownloadFolder_forRemoteWebdriver() {
    config.headless(true);
    config.remote("https://some.remote.blah:1234/wd");

    FirefoxOptions options = driverFactory.createCapabilities(config, browser, proxy, null);

    Map<String, Object> prefs = prefs(options);
    assertThat(prefs.get("browser.download.dir")).isNull();
    assertThat((String) prefs.get("browser.helperApps.neverAsk.saveToDisk")).contains("application/pdf");
    assertThat(prefs.get("pdfjs.disabled")).isEqualTo(true);
    assertThat(prefs.get("browser.download.folderList")).isEqualTo(2);
  }

  @Test
  public void injectPrefs() {
    FirefoxOptions firefoxOptions = new FirefoxOptions();
    firefoxOptions.addPreference("general.useragent.override", "my agent");
    firefoxOptions.addPreference("boolean pref", true);
    firefoxOptions.addPreference("int pref", 10);
    config.browserCapabilities(firefoxOptions);

    Map<String, Object> options = driverFactory.createCapabilities(config, browser, proxy, null).asMap();
    assertThat(options.get("moz:firefoxOptions")).isNotNull();

    Map<String, Object> prefs = (Map<String, Object>) ((Map<String, Object>) options.get("moz:firefoxOptions")).get("prefs");
    assertThat(prefs.get("general.useragent.override")).isEqualTo("my agent");
    assertThat(prefs.get("boolean pref")).isEqualTo(true);
    assertThat(prefs.get("int pref")).isEqualTo(10);
  }

  @Test
  void canActivatePDFviewer() {
    FirefoxOptions firefoxOptions = new FirefoxOptions();
    firefoxOptions.addPreference("pdfjs.disabled", false);
    config.browserCapabilities(firefoxOptions);

    FirefoxOptions firefoxOptionsMerged = driverFactory.createCapabilities(config, browser, proxy, browserDownloadsFolder);

    Map<String, Object> prefsMap = getBrowserLaunchPrefs(FirefoxOptions.FIREFOX_OPTIONS, firefoxOptionsMerged);

    assertThat(prefsMap).containsEntry("pdfjs.disabled", false);
  }

  @SuppressWarnings("unchecked")
  private Map<String, Object> prefs(FirefoxOptions options) {
    Map<String, Object> firefoxOptions = firefoxOptions(options);
    return (Map<String, Object>) firefoxOptions.get("prefs");
  }

  @SuppressWarnings("unchecked")
  private Map<String, Object> firefoxOptions(FirefoxOptions options) {
    return (Map<String, Object>) options.asMap().get("moz:firefoxOptions");
  }
}
